#ifndef _m047_lib_
#define _m047_lib_
/***************************************************************************/
/**                                                                       **/
/**              MODULAR-4 library for SPB module M-C16-3                 **/
/**                                                                       **/
/**                             Version 2.A.                              **/
/**                                                                       **/
/**             Last change      : 29.08.1997 				  **/
/**                                                                       **/
/**  This file contains routines for Borland C++/3.1 to allow the user,   **/
/**  to write programs for the MODULAR-4 in "C" 			  **/
/**                                                                       **/
/**                         Author : H.K.                                 **/
/**                         Status : 					  **/
/**                                                                       **/
/**                                                                       **/
/** Special comments: 							  **/
/**                                                                       **/
/** This library was tested with Borland C++ Version 3.1.	          **/
/**                                                                       **/
/** No parameters are checked.						  **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**               (c) SORCUS Computer GmbH 1997                           **/
/**                                                                       **/
/***************************************************************************/

/* The switches for the library are set in the configuration files of Pascal
   (*.TP file) and C (project-file). They can also be set in this source file:

   Then the following #defines have to exist:

   If you use Pascal, #define BORLAND_PASCAL has to be included (default is C).

   If the library shall be used as realtime library on the MODULAR-4/486
   #define ML8 and #define ML8RT have to be included (default is PC library).

   If you work with Windows, this is detected by the compiler. No #defines have
	to be included.
*/

#define MBASIS               micro_slot_basis[micro_slot] /* Modul-Basisadresse */

/****************** Start of register definition **********************************/

/* Bus-Interface */

#define M047_RSR_DIRECT      28      // Register-Select-Register
#define M047_IDR_DIRECT      0       // Indirect-Data-Register
#define M047_MIR_INDIRECT    3       // Modul-Interface-Register,    RSR = 3
#define M047_CVR_INDIRECT    11      // Chip-Version-Register,       RSR = 11
#define M047_ISR_INDIRECT    19      // Interrupt-Select-Register,   RSR = 19
#define M047_MSR_INDIRECT    123     // Modul-Status-Register,       RSR = 123

/* External Inputs */

#define M047_IIR_INDIRECT    27      // Input-Invert-Register,       RSR = 27
#define M047_IRR_DIRECT      24      // Input-Read-Register
#define M047_ICR_INDIRECT    43      // Input-Command-Register,      RSR = 43
#define M047_ILR_INDIRECT    51      // Input-Latch-Register,        RSR = 51

/* Externe Outputs */

#define M047_OMR_INDIRECT    67      // Output-Mode-Register,        RSR = 67
#define M047_ODR_DIRECT      20      // Output-Direct-Register

/* Interrupt-Controller */

#define M047_IVR_DIRECT      28      // Interrupt-Vector-Register
#define M047_IMR_INDIRECT    75      // Interrupt-Mask-Register,     RSR = 75
#define M047_IPR_INDIRECT    91      // Interrupt-Pending-Register,  RSR = 91
#define M047_IOR_INDIRECT    107     // Interrupt-Overrun-Register,  RSR = 107
#define M047_IER_INDIRECT    107     // Interrupt-Edge-Register,     RSR = 107

/* Counter 0 */

#define M047_CMR0_INDIRECT       0  	 // Counter-Mode-Register,       RSR = 0
#define M047_CIR0_LOW_INDIRECT   8       // Counter-Input-Register,      RSR = 8
					 // Bit-0 to Bit-15
#define M047_CIR0_HIGH_INDIRECT  16      // Counter-Input-Register,      RSR = 16
					 // Bit-16 to Bit-31
#define M047_CPR0_DIRECT         4       // Counter-Preset-Register
#define M047_CDR0_DIRECT         4       // Counter-Data-Register
#define M047_CRR0_INDIRECT       24      // Counter-Referenc-Register,   RSR = 24
#define M047_CSR0_INDIRECT       24      // Counter-Status-Register,     RSR = 24

/* Counter 1 */

#define M047_CMR1_INDIRECT       1       // Counter-Mode-Register,       RSR = 1
#define M047_CIR1_LOW_INDIRECT   9       // Counter-Input-Register,      RSR = 9
					 // Bit-0 to Bit-15
#define M047_CIR1_HIGH_INDIRECT  17      // Counter-Input-Register,      RSR = 17
					 // Bit-16 to Bit-31
#define M047_CPR1_DIRECT         8       // Counter-Preset-Register
#define M047_CDR1_DIRECT         8       // Counter-Data-Register
#define M047_CRR1_INDIRECT       25      // Counter-Referenc-Register,   RSR = 25
#define M047_CSR1_INDIRECT       25      // Counter-Status-Register,     RSR = 25

/* Counter 2 */

#define M047_CMR2_INDIRECT       2       // Counter-Mode-Register,       RSR = 2
#define M047_CIR2_LOW_INDIRECT   10      // Counter-Input-Register,      RSR = 10
					 // Bit-0 to Bit-15
#define M047_CIR2_HIGH_INDIRECT  18      // Counter-Input-Register,      RSR = 18
					 // Bit-16 to Bit-31
#define M047_CPR2_DIRECT         12      // Counter-Preset-Register
#define M047_CDR2_DIRECT         12      // Counter-Data-Register
#define M047_CRR2_INDIRECT       26      // Counter-Referenc-Register,   RSR = 26
#define M047_CSR2_INDIRECT       26      // Counter-Status-Register,     RSR = 26

/* All Counters */

#define M047_CCR_DIRECT          16      // Counter-Command-Register

/****************** End of register definition **********************************/

#define M047_MIR_RESET		 0x80	 // Modul-Reset

#define M047_NO_IRQ		 0	 // No interrupt line selected
#define M047_IRQ_A		 1       // Interrupt line IRQ-A selected
#define M047_IRQ_B		 2       // Interrupt line IRQ-B selected
#define M047_IRQ_C		 3       // Interrupt line IRQ-C selected
#define M047_IRQ_D		 4       // Interrupt line IRQ-D selected
#define M047_IRQ_E		 5       // Interrupt line IRQ-E selected
#define M047_IRQ_F		 6       // Interrupt line IRQ-F selected
#define M047_IRQ_NMI		 7       // Interrupt line NMI selected
#define M047_IRQ_POS_EDGE	 0	 // Positive edge releases interrupt
#define M047_IRQ_NEG_EDGE	 16	 // Negative edge releases interrupt

#define M047_DEMASK_INT_0        1       // Demask interrupt line INT-0
#define M047_DEMASK_INT_1        2       // Demask interrupt line INT-1
#define M047_DEMASK_INT_2        4       // Demask interrupt line INT-2
#define M047_DEMASK_INT_3        8       // Demask interrupt line INT-3
#define M047_DEMASK_INT_4        16      // Demask interrupt line INT-4
#define M047_DEMASK_INT_5        32      // Demask interrupt line INT-5
#define M047_DEMASK_INT_6        64      // Demask interrupt line INT-6
#define M047_DEMASK_INT_7        128     // Demask interrupt line INT-7
#define M047_DEMASK_INT_8        256     // Demask interrupt line INT-8
#define M047_DEMASK_INT_9        512     // Demask interrupt line INT-9
#define M047_DEMASK_INT_10       1024    // Demask interrupt line INT-10
#define M047_DEMASK_INT_11       2048    // Demask interrupt line INT-11
#define M047_DEMASK_INT_12       4096    // Demask interrupt line INT-12
#define M047_DEMASK_INT_13       8192    // Demask interrupt line INT-13
#define M047_DEMASK_INT_14       16384   // Demask interrupt line INT-14
#define M047_DEMASK_INT_15       32768L  // Demask interrupt line INT-15

#define M047_INT_0_NEG_EDGE      1       // Negative edge for INT-0
#define M047_INT_1_NEG_EDGE	 2       // Negative edge for INT-1
#define M047_INT_2_NEG_EDGE	 4       // Negative edge for INT-2
#define M047_INT_3_NEG_EDGE	 8       // Negative edge for INT-3
#define M047_INT_4_NEG_EDGE	 16      // Negative edge for INT-4
#define M047_INT_5_NEG_EDGE	 32      // Negative edge for INT-5
#define M047_INT_6_NEG_EDGE	 64      // Negative edge for INT-6
#define M047_INT_7_NEG_EDGE	 128     // Negative edge for INT-7
#define M047_INT_8_NEG_EDGE	 256     // Negative edge for INT-8
#define M047_INT_9_NEG_EDGE	 512     // Negative edge for INT-9
#define M047_INT_10_NEG_EDGE     1024    // Negative edge for INT-10
#define M047_INT_11_NEG_EDGE     2048    // Negative edge for INT-11
#define M047_INT_12_NEG_EDGE     4096    // Negative edge for INT-12
#define M047_INT_13_NEG_EDGE     8192    // Negative edge for INT-13
#define M047_INT_14_NEG_EDGE     16384   // Negative edge for INT-14
#define M047_INT_15_NEG_EDGE     32768L  // Negative edge for INT-15

#define M047_LATCH_POS_EDGE	 0	 // Positive edge for latch input
#define M047_LATCH_NEG_EDGE	 32	 // Negative edge for latch input
#define M047_LATCH_DISABLE	 0	 // Disable latch input
#define M047_LATCH_ENABLE	 64	 // Enable latch input

#define M047_INACTIVE		 0	 // Counter not activated
#define M047_COUNTER_UP	 	 1	 // Up-Counter
#define M047_COUNTER_DN		 2 	 // Down-Counter
#define M047_COUNTER_UP_DN_A	 3	 // Up-/Down-Counter type A
#define M047_COUNTER_UP_DN_B	 4	 // Up-/Down-Counter type B
#define M047_TIMER		 5	 // Timer
#define M047_INCREMENT_A	 6	 // Incremental interface type A
#define M047_INCREMENT_B	 7	 // Incremental interface type B
#define M047_FREQUENCY		 9	 // Frequency measurement
#define M047_PULSEWIDTH		 10	 // Pulsewidth measurement
#define M047_PERIOD		 11	 // Period measurement
#define M047_VELOCITY		 12	 // Velocity measurement
#define M047_MULTIPLE_PERIOD	 4096	 // Continuous period measurement
#define M047_COUNTER		 4096 	 // Additional Counter

#define M047_AUTOSTART		 32	 // Autostart
#define M047_CONTINUE		 64	 // Continue (auto. reload after zero crossing)
#define M047_EXTERN		 128	 // Count gate from external
#define M047_SINGLE_A		 0	 // Incremental interface: Mode Single Phase A
#define M047_SINGLE_B		 64	 // Incremental interface: Mode Single Phase B
#define M047_DOUBLE_A		 128	 // Incremental interface: Mode Double Phase A
#define M047_DOUBLE_B		 192	 // Incremental interface: Mode Double Phase B
#define M047_QUADRUPLE		 256	 // Incremental interface: Mode Quadruple
#define	M047_AUTOFREEZE		 256	 // Auto. freeze after end of measurement
#define M047_AUTORESET		 512	 // Auto. reset after end of measurement
#define M047_AUTOLOAD		 1024	 // Auto. load after ende of measurement
#define M047_OVERRIDE		 2048	 // Old counter value is overwritten by freeze
#define M047_CASCADE		 32768L	 // Cascade to previous counters
#define M047_CASCADE_0_1	 8192	 // Cascade of counter 0 and 1
#define M047_CASCADE_1_2	 16384	 // Cascadr of counter 1 and 2
#define M047_CASCADE_0_1_2	 32768L	 // Cascade of counter 0, 1 and 2

#define M047_DIN_0_1_CNT_0	 0       // DIN-0 to count input 0 of counter 0
#define M047_DIN_2_3_CNT_0	 4096    // DIN-2 to count input 0 of counter 0
#define M047_DIN_4_5_CNT_0	 8192    // DIN-4 to count input 0 of counter 0
#define M047_DIN_6_7_CNT_0	 12288   // DIN-6 to count input 0 of counter 0
#define M047_DIN_8_9_CNT_0	 16384   // DIN-8 to count input 0 of counter 0
#define M047_DIN_10_11_CNT_0	 20480   // DIN-10 to count input 0 of counter 0
#define M047_DIN_6_CNT_1	 24576   // DIN-6 to count input 0 of counter 0
#define M047_CNT_0_CNT_1	 28672   // Count input 0 of counter 1
					 // to count input 1 of counter 0
#define M047_DIN_4_CNT_2	 32768L  // DIN-4 to count input 0 of counter 0
#define M047_CNT_0_CNT_2	 36864L  // Count input 0 of counter 2
					 // to count input 0 of counter 0

#define M047_DIN_0_INV		 1	 // Invert input signal at DIN-0
#define M047_DIN_1_INV		 2	 // Invert input signal at DIN-1
#define M047_DIN_2_INV		 4	 // Invert input signal at DIN-2
#define M047_DIN_3_INV		 8       // Invert input signal at DIN-3
#define M047_DIN_4_INV		 16	 // Invert input signal at DIN-4
#define M047_DIN_5_INV		 32      // Invert input signal at DIN-5
#define M047_DIN_6_INV		 64	 // Invert input signal at DIN-6
#define M047_DIN_7_INV		 128     // Invert input signal at DIN-7
#define M047_DIN_8_INV		 256     // Invert input signal at DIN-8
#define M047_DIN_9_INV		 512     // Invert input signal at DIN-9
#define M047_DIN_10_INV		 1024    // Invert input signal at DIN-10
#define M047_DIN_11_INV		 2048    // Invert input signal at DIN-11

#define M047_NO_DOUT             0 	 // No output of counter
#define M047_DOUT_0_CNT_0	 1 	 // DOUT-0 is output of counter 0
#define M047_DOUT_1_CNT_1	 2 	 // DOUT-1 is output of counter 1
#define M047_DOUT_2_CNT_2	 4 	 // DOUT-2 is output of counter 2

#define M047_NO_START		 0	 // Start not by external input
#define M047_START_DIN_0	 16	 // Start by external input DIN-0
#define M047_START_DIN_1	 32	 // Start by external input DIN-1
#define M047_START_DIN_2	 48	 // Start by external input DIN-2

#define M047_NO_STOP		 0	 // Stop not by external input
#define M047_STOP_DIN_1		 64	 // Stop by external input DIN-1
#define M047_STOP_DIN_2		 128	 // Stop by external input DIN-2
#define M047_STOP_DIN_3	 	 192	 // Stop by external input DIN-3
#define M047_NO_STOP_UFL	 0	 // Stop not by underflow of counter 0
#define M047_STOP_UFL_CNT_0	 256	 // Stop by underflow of counter 0
#define M047_STOP_UFL_CNT_1	 512	 // Stop by underflow of counter 1
#define M047_STOP_UFL_CNT_2	 768	 // Stop by underflow of counter 2

#define M047_NO_LOAD		 0	 // Load not by external input
#define M047_LOAD_DIN_0	 	 1024	 // Load by external input DIN-0
#define M047_LOAD_DIN_1	 	 2048	 // Load by external input DIN-1
#define M047_LOAD_DIN_2	 	 3072	 // Load by external input DIN-2

#define M047_NO_FREEZE		 0	 // Freeze not by external input
#define M047_FREEZE_DIN_1 	 4096	 // Freeze by external input DIN-1
#define M047_FREEZE_DIN_2 	 8192	 // Freeze by external input DIN-2
#define M047_FREEZE_DIN_3 	 12288	 // Freeze by external input DIN-3

#define M047_NO_RESET		 0	 // Reset not by external input
#define M047_RESET_DIN_0 	 16384	 // Reset by external input DIN-0
#define M047_RESET_DIN_1 	 32768L	 // Reset by external input DIN-1
#define M047_RESET_DIN_2 	 49152L	 // Reset by external input DIN-2

#define M047_NO_INT              0 	 // No interrupt
#define M047_INT_CNT_0_UFL	 1	 // Release interrupt by underflow counter 0
#define M047_INT_CNT_0_OVL	 2	 // Release interrupt by overflow counter 0
#define M047_INT_CNT_0_END	 4	 // Release interrupt by end of measurement
#define M047_INT_CNT_0_INVALID	 8	 // Release interrupt by invalid measurement
#define M047_INT_CNT_1_UFL	 16	 // Release interrupt by underflow counter 1
#define M047_INT_CNT_1_OVL	 32	 // Release interrupt by overflow counter 1
#define M047_INT_CNT_1_END	 64	 // Release interrupt by end of measurement
#define M047_INT_CNT_2_UFL	 128	 // Release interrupt by underflow counter 2
#define M047_INT_CNT_2_OVL	 256	 // Release interrupt by overflow counter 2
#define M047_INT_CNT_2_END	 512	 // Release interrupt by end of measurement

#define M047_NO_FREQ		 0	 // No reference frequency
#define M047_10_MHZ		 2	 // Reference frequency 10 MHz
#define M047_5_MHZ		 3       //                      5 MHz
#define M047_2_MHZ		 4       //                      1 MHz
#define M047_1_MHZ		 5       //                         :
#define M047_500_KHZ		 6       //                         :
#define M047_200_KHZ		 7
#define M047_100_KHZ		 8
#define M047_50_KHZ		 9
#define M047_20_KHZ		 10
#define M047_10_KHZ		 11
#define M047_5_KHZ		 12
#define M047_2_KHZ		 13
#define M047_1_KHZ		 14
#define M047_500_HZ		 15
#define M047_200_HZ		 16
#define M047_100_HZ		 17
#define M047_50_HZ		 18
#define M047_20_HZ		 19
#define M047_10_HZ		 20

#define M047_NO_PERIOD		 0
#define M047_100_NS		 2
#define M047_200_NS              3
#define M047_500_NS              4
#define M047_1_US		 5
#define M047_2_US              	 6
#define M047_5_US              	 7
#define M047_10_US		 8
#define M047_20_US               9
#define M047_50_US               10
#define M047_100_US		 11
#define M047_200_US              12
#define M047_500_US              13
#define M047_1_MS		 14
#define M047_2_MS                15
#define M047_5_MS                16
#define M047_10_MS		 17
#define M047_20_MS               18
#define M047_50_MS               19
#define M047_100_MS		 20

#define M047_NO_MULTIPLE         0      // No multiple measurement

#define M047_CMD_START_COUNTER   1
#define M047_CMD_STOP_COUNTER    2
#define M047_CMD_LOAD_COUNTER    4
#define M047_CMD_FREEZE_COUNTER  8
#define M047_CMD_RESET_COUNTER   16

#define M047_COUNTER_0		 1
#define M047_COUNTER_1		 2
#define M047_COUNTER_2		 4
#define M047_COUNTER_0_1	 3
#define M047_COUNTER_1_2	 6
#define M047_COUNTER_0_2	 5
#define M047_COUNTER_0_1_2	 7


#include <module.h>

typedef ushort m047_block[3];
typedef struct {
	ushort	IndexFirst;
	ushort	IndexLast;
	ushort 	Mode;
	ushort	CounterInput;
	ushort	CounterOutput;
	ushort	CommandInput;
	ushort	Interrupt;
	ushort	Time;
	ushort	Multiple;
	} m047_config_struct;

#if defined(__cplusplus)
extern "C" {
#endif


/* Init module library
*/
MLXAPI void EXPORT m047_bib_startup (byte micro_slot);

/* Config module as defined in EEPROM
*/
MLXAPI void EXPORT m047_set_conf_eeprom (byte micro_slot);

/* Select interrupt line to base board
*/
MLXAPI void EXPORT m047_select_irq (byte micro_slot, byte irqchannel, byte irqedge);

/* Get selected interrupt line to base board
*/
MLXAPI void EXPORT m047_get_selected_irq (byte micro_slot, byte *irqchannel, byte *irqedge);

/* Mask/demask interrupt inputs
*/
MLXAPI void EXPORT m047_mask_interrupt (byte micro_slot, ushort intchannel);

/* Define edge of interrupt inputs
*/
MLXAPI void EXPORT m047_set_int_edge (byte micro_slot, ushort intedge);

/* Get masked interrupt inputs and corresponding edges
*/
MLXAPI void EXPORT m047_get_interrupt (byte micro_slot, ushort *intchannel, ushort *intedge);

/* Define latch input
*/
MLXAPI void EXPORT m047_define_latch_input (byte micro_slot, byte channel, byte edge, byte enable);

/* Get define latch input
*/
MLXAPI void EXPORT m047_get_latch_input (byte micro_slot, byte *channel, byte *edge, byte *enable);

/* Define output function
*/
MLXAPI void EXPORT m047_define_output (byte micro_slot, byte outputfunc);

/* Get defined output function
*/
MLXAPI void EXPORT m047_get_outputfunc (byte micro_slot, byte *outputfunc);

/* Configure counter
*/
MLXAPI void EXPORT m047_counter_config (byte micro_slot, m047_config_struct *config);

/* Send command to counter
*/
MLXAPI void EXPORT m047_send_command (byte micro_slot, ushort command, byte counter);

/* Write word to counter
*/
MLXAPI void EXPORT m047_preset_counter_word (byte micro_slot, byte counter, ushort data);

/* Write dword to counter
*/
MLXAPI void EXPORT m047_preset_counter_dword (byte micro_slot, byte counter, ulong data);

/* Write block to counter
*/
MLXAPI void EXPORT m047_preset_counter_block (byte micro_slot, byte counter, m047_block data);

/* Read word from counter
*/
MLXAPI ushort EXPORT m047_read_counter_word (byte micro_slot, byte counter);

/* Read double word from counter
*/
MLXAPI ulong EXPORT m047_read_counter_dword (byte micro_slot, byte counter);

/* Read block from counter
*/
MLXAPI void EXPORT m047_read_counter_block (byte micro_slot, byte counter, m047_block data);

/* Read status of counter
*/
MLXAPI byte EXPORT m047_read_counter_status (byte micro_slot, byte counter);

/* Read digital inputs
*/
MLXAPI ushort EXPORT m047_read_digital_input (byte micro_slot);

/* Read digital inputs from latch
*/
MLXAPI ushort EXPORT m047_read_digital_input_latched (byte micro_slot);

/* Latch digital inputs
*/
MLXAPI void EXPORT m047_latch_digital_input (byte micro_slot);

/* Set digital outputs
*/
MLXAPI void EXPORT m047_set_digital_output (byte micro_slot, byte data);

/* Read interrupt-vector-register
*/
MLXAPI void EXPORT m047_read_ivr (byte micro_slot, byte *intnumber, byte *overrun);

/* Read interrupt-pending-register
*/
MLXAPI ushort EXPORT m047_read_ipr (byte micro_slot);

/* Read interrupt-overrun-register
*/
MLXAPI ushort EXPORT m047_read_ior (byte micro_slot);


#if defined(__cplusplus)
} /*  End of block extern "C" */
#endif


#endif
