program c16beispiel_3;
(***************************************************************************)
(* Beispielprogramm C16BSP_3.PAS in BORLAND PASCAL 7.0 fr die Verwendung  *)
(* des Moduls M-C16-3.                                                     *)
(*                                                                         *)
(* Version von C16BSP_3.PAS: 1.A.    Stand: 29.08.1997                     *)
(*                                                                         *)
(* Dieses Beispielprogramm demonstriert den Einsatz des M-C16-3 fr die    *)
(* Periodendauermessung (Aufloesung 32 Bit, Referenz=10 MHz                *)
(***************************************************************************)

(***************************************************************************)
(********************************   USES   *********************************)
(***************************************************************************)

uses
    crt,ml8bib,ml8error,m047_lib;

(***************************************************************************)
(******************************** DEFINES **********************************)
(***************************************************************************)

CONST
     BAdr = $380;                  (* Basisadresse der MODULAR-4/486-Karte *)
     SLOT = 2;                     (* Steckplatz des M-C16-3-Moduls        *)
     COUNTER_0 = 0;                (* Zaehler 0                            *)
     COUNTER_1 = 1;                (* Zaehler 1                            *)
     OSX : str80 = 'ROM';

(***************************************************************************)
(***************************** HAUPTPROGRAMM *******************************)
(***************************************************************************)

var
   taste: char;                 (* Tastatur-Input                   *)
   cdr0: word;                  (* Zaehlerstand des Zaehlerkanals 0 *)
				(* (Counter-Data-Register Nr. 0)    *)
   cdr1: word;                  (* Zaehlerstand des Zaehlerkanals 1 *)
				(* (Counter-Data-Register Nr. 1)    *)
   result: longint;             (* Periodendauer *)
   configuration: m047_config_struct; (* Konfigurationsstruktur     *)

begin

(* Programmiersprache = PASCAL *)

  ml8_bib_startup (LANGUAGE_PASCAL);

(* Basiskarte zuruecksetzen *)

  ml8_reset(0, BAdr, 0, 7, 0, @error_handler, OSX, 10);

(* Initialisierung des M-C16-3 *)

  clrscr;
  writeln('BEISPIEL: Einsatz des M-C16-3 fuer 32-Bit-Periodendauermessung');
  writeln('1. Schliessen sie bitte das zu messende Signal an DIN-7 an.');
  writeln('2. Stecken Sie das M-C16-3-Modul in Slot 2.');
  writeln('Einstellungen: Referenzfrequenz = 10MHz, Aufloesung = 32 Bit');

(* M-C16-3 zuruecksetzen *)

  m047_bib_startup(SLOT);


(* Einsatz des M-C16-3 zur Periodendauermessung 32-Bit *)

  configuration.IndexFirst := 0;
  configuration.IndexLast := 1;

(* Zaehlerkanal 0 in den Mode "Periodendauermessung" und in die Submodes
   CONTINUE, AUTOFREEZE und AUTORESET versetzen *)

  configuration.Mode := M047_PERIOD+
			M047_AUTORESET+
			M047_AUTOFREEZE+
			M047_CONTINUE+

(* Zaehlerkanal 1 kaskadieren *)

			M047_CASCADE_0_1;

(* Die Zaehleingaenge des Zaehlers 0 sind nach einem Reset mit DIN-8
   und DIN-9 verbunden. Hier werden die Zaehleingnge nun an DIN-6
   und DIN-7 umgeleitet. *)

  configuration.CounterInput := M047_DIN_4_5_CNT_0;

(* Kein Zaehlerausgang *)

  configuration.CounterOutput := M047_NO_DOUT;

  configuration.CommandInput := M047_NO_START+
				M047_NO_STOP+
				M047_NO_STOP_UFL+
				M047_NO_LOAD+
				M047_NO_FREEZE+
				M047_NO_RESET;
(* Kein Interrupt *)

  configuration.Interrupt := M047_NO_INT;

(* Referenz-Frequenz (10 MHz) einstellen *)

  configuration.Time := M047_10_MHZ;

(* Keine Mehrfachmessung *)

  configuration.Multiple := 0;

(* Zaehler konfigurieren *)

  m047_counter_config(SLOT,configuration);

  gotoxy (1,9);

  writeln('1 = Messung starten');
  writeln('2 = Messung stoppen');
  writeln('3 = Zaehler zuruecksetzen');
  writeln('ESC = Ende');

  taste := '0';

  repeat
    gotoxy (1,12);
    writeln('Bitte waehlen Sie: ');

    if keypressed then taste := readkey;

    case taste of
      '1': begin
	     (* Zaehler starten *)
	     m047_send_command(SLOT,M047_CMD_START_COUNTER,M047_COUNTER_0);
	     taste := '0';
	   end;
      '2': begin
	     (* Zaehler stoppen *)
	     m047_send_command(SLOT,M047_CMD_STOP_COUNTER,M047_COUNTER_0);
	     taste := '0';
	   end;
      '3': begin
	     (* Zaehler zuruecksetzen *)
	     m047_send_command(SLOT,M047_CMD_RESET_COUNTER,M047_COUNTER_0);
	     (* Zaehlerstand 0 in Ausgangsregister cdr0 und cdr1
		uebernehmen *)
	     m047_send_command(SLOT,M047_CMD_FREEZE_COUNTER,M047_COUNTER_0);
	     taste := '0';
	   end;
      end;

  (* Bit 0 bis 15 des Ergebnisses lesen *)

    cdr0 := m047_read_counter_word(SLOT,COUNTER_0);

  (* Bit 16 bis 32 des Ergebnisses lesen *)

    cdr1 := m047_read_counter_word(SLOT,COUNTER_1);

    gotoxy(1,18);

  (* Ausgabe des Messwertes *)

    result := ((cdr1*$10000)+cdr0) div 10;
    gotoxy (1,20);
    writeln('Periodendauer: ',result:8,' us');

  until taste = #27;

  ml8_exit;
end.

